//!	EOS_Event.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_Event_h_
#define _EOS_Event_h_


//--------------------------- include section -----------------------
#include "EOS_EventHandler.h"
#include "EOS_List.h"


//--------------------------- class definition ----------------------
//! A CEOS_Event is a class, which calls attached event-handlers when
//! being triggerd.
//! This class is a template class with 10 arguments, specialized
//! template versions exists down to 1 argument and even no argument
//! \code
//! CEOS_Event<double,const char*> DoubleCharEvent;
//! DoubleCharEvent.trigger(1.23,"Hallo World");
//!
//! CEOS_Event<CWhatEverObject*> ObjectEvent;
//! CWhatEverObject WhatEverObject;
//! ObjectEvent.trigger(&WhatEverObject);
//! \endcode
template <typename A1=void,
          typename A2=void,
          typename A3=void,
          typename A4=void,
          typename A5=void,
          typename A6=void,
          typename A7=void,
          typename A8=void,
          typename A9=void,
          typename A10=void>
class CEOS_Event
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    //! Adds a function to the event, which is called, when the
    //! function trigger is called.
    //! The attached function must have the same number and types of
    //! arguments like the event itself. Otherwise the compiler will
    //! generate an error
    //! \code
    //! CEOS_Event<int,double,const char*,void*> Event;
    //!
    //! void handleEventShort(int,double);
    //! void handleEventRight(int,double,const char*,void*);
    //!
    //! void initialize()
    //! {
    //!     Event.add(handleEventShort);   // Error: too few arguments
    //!     Event.add(handleEventRight);   // Ok
    //! }
    //! \endcode
    void add(void(pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    //! Adds a function to the event, which is called, when the
    //! function trigger is called.
    //! The attached function must have the same number and types of
    //! arguments like the event itself. Otherwise the compiler will
    //! generate an error
    //! \code
    //! CEOS_Event<int,double,const char*,void*> Event;
    //!
    //! void CMyClass::handleEventShort(int,double);
    //! void CMyClass::handleEventRight(int,double,const char*,void*);
    //!
    //! void CMyClass::initialize()
    //! {
    //!     Event.add(this,&CMyClass::handleEventShort);   // Error: too few arguments
    //!     Event.add(this,&CMyClass::handleEventRight);   // Ok
    //! }
    //! \endcode
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! Will call all attached event handlers.
    //! The number and type of arguments must match to the given
    //! arguments by the templates
    //! \code
    //! CEOS_Event<int,double,const char*,void*> Event;
    //!
    //! Event.trigger(1,2.0,"Hallo",NULL); // Ok
    //! Event.trigger("Hallo",123);        // Error: not the right arguments
    //! \endcode
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5,
                 A6 arg6,
                 A7 arg7,
                 A8 arg8,
                 A9 arg9,
                 A10 arg10)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers with
    //! exactly the right number and types of arguments
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8,
          typename A9>
class CEOS_Event<A1,A2,A3,A4,A5,A6,A7,A8,A9>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8,A9>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8,A9>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5,
                 A6 arg6,
                 A7 arg7,
                 A8 arg8,
                 A9 arg9)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8>
class CEOS_Event<A1,A2,A3,A4,A5,A6,A7,A8>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4,A5,A6,A7,A8))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5,
                 A6 arg6,
                 A7 arg7,
                 A8 arg8)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7>
class CEOS_Event<A1,A2,A3,A4,A5,A6,A7>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4,A5,A6,A7))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5,
                 A6 arg6,
                 A7 arg7)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5,arg6,arg7);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6>
class CEOS_Event<A1,A2,A3,A4,A5,A6>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4,A5,A6))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5,
                 A6 arg6)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5,A6>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5,arg6);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5,A6>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5>
class CEOS_Event<A1,A2,A3,A4,A5>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4,A5))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4,A5>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4,A5>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4,
                 A5 arg5)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4,A5>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4,arg5);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4,A5>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4>
class CEOS_Event<A1,A2,A3,A4>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3,A4))
    {
        CEOS_EventHandler<void,A1,A2,A3,A4>* pHandler = new CEOS_EventHandler<void,A1,A2,A3,A4>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3,A4))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3,
                 A4 arg4)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3,A4>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3,arg4);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3,A4>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3>
class CEOS_Event<A1,A2,A3>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2,A3))
    {
        CEOS_EventHandler<void,A1,A2,A3>* pHandler = new CEOS_EventHandler<void,A1,A2,A3>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2,A3))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2,A3>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2,A3>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2,
                 A3 arg3)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2,A3>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2,A3>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2,arg3);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2,A3>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2>
class CEOS_Event<A1,A2>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)(A1,A2))
    {
        CEOS_EventHandler<void,A1,A2>* pHandler = new CEOS_EventHandler<void,A1,A2>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1,A2))
    {
        CEOS_EventHandler<T_OBJECT,A1,A2>* pHandler = new CEOS_EventHandler<T_OBJECT,A1,A2>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1,
                 A2 arg2)
    {
        typename CEOS_List<CEOS_HandlerBase<A1,A2>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1,A2>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1,arg2);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1,A2>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <typename A1>
class CEOS_Event<A1>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(*pFunc)(A1))
    {
        CEOS_EventHandler<void,A1>* pHandler = new CEOS_EventHandler<void,A1>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(A1))
    {
        CEOS_EventHandler<T_OBJECT,A1>* pHandler = new CEOS_EventHandler<T_OBJECT,A1>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger(A1 arg1)
    {
        typename CEOS_List<CEOS_HandlerBase<A1>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<A1>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute(arg1);
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<A1>*> m_HandlerList;
};


//########################### specialized template ##################
//--------------------------- class definition ----------------------
template <>
class CEOS_Event<>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Event);


//--------------------------- public access -------------------------
public:
    // add ----------------------------------------------------------
    void add(void(pFunc)())
    {
        CEOS_EventHandler<void>* pHandler = new CEOS_EventHandler<void>(pFunc);
        m_HandlerList.add(pHandler);
    }

    // add ----------------------------------------------------------
    template <class T_OBJECT>
    void add(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)())
    {
        CEOS_EventHandler<T_OBJECT>* pHandler = new CEOS_EventHandler<T_OBJECT>(pObject,pFunc);
        m_HandlerList.add(pHandler);
    }

    // trigger ------------------------------------------------------
    //! informs all attached CEOS_Eventhandler objects that the event
    //! was triggered
    void trigger()
    {
        CEOS_List<CEOS_HandlerBase<>*>::TPosition pPosition = m_HandlerList.first();
        while (pPosition != NULL)
        {
            CEOS_HandlerBase<>* pHandler = m_HandlerList.get(pPosition);
            pHandler->onExecute();
            pPosition = m_HandlerList.next(pPosition);
        }
    }


//--------------------------- protected data ------------------------
protected:
    // m_HandlerList ------------------------------------------------
    //! \internal
    //! list holding the information about all attached handlers
    CEOS_List<CEOS_HandlerBase<>*> m_HandlerList;
};


#endif
